/* Tracker Extract - extracts embedded metadata from files
 * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
 * Flac support added by Michael Cronenworth (mike@cchtml.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#include "config.h"

#include <stdio.h>
#include <string.h>
#include <fcntl.h>

#include <glib.h>

#include <libmodplug/modplug.h>

/*
 * #include <libmodplug/stdafx.h>
 * #include <libmodplug/sndfile.h>
 */

/* Copied from sndfile.h */
#define MOD_TYPE_NONE           0x00
#define MOD_TYPE_MOD            0x01
#define MOD_TYPE_S3M            0x02
#define MOD_TYPE_XM             0x04
#define MOD_TYPE_MED            0x08
#define MOD_TYPE_MTM            0x10
#define MOD_TYPE_IT             0x20
#define MOD_TYPE_669            0x40
#define MOD_TYPE_ULT            0x80
#define MOD_TYPE_STM            0x100
#define MOD_TYPE_FAR            0x200
#define MOD_TYPE_WAV            0x400
#define MOD_TYPE_AMF            0x800
#define MOD_TYPE_AMS            0x1000
#define MOD_TYPE_DSM            0x2000
#define MOD_TYPE_MDL            0x4000
#define MOD_TYPE_OKT            0x8000
#define MOD_TYPE_MID            0x10000
#define MOD_TYPE_DMF            0x20000
#define MOD_TYPE_PTM            0x40000
#define MOD_TYPE_DBM            0x80000
#define MOD_TYPE_MT2            0x100000
#define MOD_TYPE_AMF0           0x200000
#define MOD_TYPE_PSM            0x400000
#define MOD_TYPE_J2B            0x800000
#define MOD_TYPE_ABC            0x1000000
#define MOD_TYPE_PAT            0x2000000
#define MOD_TYPE_UMX            0x80000000 // Fake type
#define MAX_MODTYPE             24


#include "tracker-ripped-functions.h"
#include "tracker-extract-module-api.h"

static void extract_mod (const char *filename,
                          GHashTable *metadata);

static TrackerExtractData extract_data[] = {
	{ "audio/x-mod", extract_mod },
	{ "audio/x-s3m", extract_mod },
	{ "audio/x-xm", extract_mod },
	{ "audio/x-it", extract_mod },
	{ "audio/x-stm", extract_mod },
	{ NULL, NULL }
};


static void
extract_mod (const char *filename, GHashTable *metadata)
{
gchar *title, *data, *comments;
gint len, mtype;
gsize dlen;
guint ch;

ModPlugFile *mod;

if (!g_file_get_contents(filename, &data, &dlen, NULL))
	return;

mod=ModPlug_Load(data, dlen);

if (!mod)
	return;

title=ModPlug_GetName(mod);
len=ModPlug_GetLength(mod);
ch=ModPlug_NumChannels(mod);
comments=ModPlug_GetMessage(mod);
mtype=ModPlug_GetModuleType(mod);

/* Modules can mostly only contain a title,
   the sample names have usually been used as artist info,
   but we can't parse that so fake something so they are nicely grouped.
 */
if (title)
	g_hash_table_insert(metadata, g_strdup ("Audio:Title"), g_strdup(title));
g_hash_table_insert(metadata, g_strdup ("Audio:Artist"), g_strdup("Various Artists"));
g_hash_table_insert(metadata, g_strdup ("Audio:Album"), g_strdup("Tracker Modules"));
g_hash_table_insert(metadata, g_strdup ("Audio:Codec"), g_strdup("modplug"));
g_hash_table_insert(metadata, g_strdup ("Audio:Samplerate"), g_strdup("44100"));
g_hash_table_insert(metadata, g_strdup ("Audio:Duration"), g_strdup_printf("%d", len/1000));
if (ch>0)
	g_hash_table_insert(metadata, g_strdup ("Audio:Channels"), g_strdup_printf("%d", ch));
if (comments)
	g_hash_table_insert(metadata, g_strdup ("Audio:Comment"), g_strdup(comments));

/* There is no genre, but fake so you can browse by type instead */
switch(mtype) {
	case MOD_TYPE_MOD:
		g_hash_table_insert(metadata, g_strdup ("Audio:Genre"), g_strdup("Protracker module"));
	break;
	case MOD_TYPE_S3M:
		g_hash_table_insert(metadata, g_strdup ("Audio:Genre"), g_strdup("ScreamTracker III module"));
	break;
	case MOD_TYPE_XM:
		g_hash_table_insert(metadata, g_strdup ("Audio:Genre"), g_strdup("Fasttracker II module"));
	break;
	case MOD_TYPE_IT:
		g_hash_table_insert(metadata, g_strdup ("Audio:Genre"), g_strdup("Impulse Tracker module"));
	break;
	case MOD_TYPE_669:
		g_hash_table_insert(metadata, g_strdup ("Audio:Genre"), g_strdup("Composer 669 module"));
	break;
	case MOD_TYPE_STM:
		g_hash_table_insert(metadata, g_strdup ("Audio:Genre"), g_strdup("Scream Tracker II module"));
	break;
	case MOD_TYPE_OKT:
		g_hash_table_insert(metadata, g_strdup ("Audio:Genre"), g_strdup("Oktalyzer module"));
	break;
	case MOD_TYPE_MED:
		g_hash_table_insert(metadata, g_strdup ("Audio:Genre"), g_strdup("Med"));
	break;
	case MOD_TYPE_MTM:
		g_hash_table_insert(metadata, g_strdup ("Audio:Genre"), g_strdup("MultiTracker module"));
	break;
	case MOD_TYPE_ULT:
		g_hash_table_insert(metadata, g_strdup ("Audio:Genre"), g_strdup(""));
	break;
	case MOD_TYPE_FAR:
		g_hash_table_insert(metadata, g_strdup ("Audio:Genre"), g_strdup(""));
	break;
	default: ;
}

ModPlug_Unload(mod);
g_free(data);
}

TrackerExtractData *
tracker_get_extract_data (void)
{
return extract_data;
}
